import type {paths, components} from './schema';
import createClient from 'openapi-fetch';
import {fetchEventSource} from '@microsoft/fetch-event-source';

const client = createClient<paths>();
client.use({
    onRequest({ schemaPath, request }) {
        if (schemaPath.startsWith('/api/public') || rpc.token == '')
            return;

        request.headers.set('Authorization', `Bearer ${rpc.token}`);
        return request;
    }
})

export type Session = components['schemas']['de.mattv.fileserver.Response$Session'];
export type Node = components['schemas']['de.mattv.fileserver.Response$Node'];
export type PathSegment = components['schemas']['de.mattv.fileserver.Response$PathSegment'];
export type CreateNodeInfo = components['schemas']['de.mattv.fileserver.Response$CreateNodeInfo'];
export type UserInfo = components['schemas']['de.mattv.fileserver.Response$UserInfo'];

export const rpc = {
    token: '',

    signup: (username: string, password: string) =>
        client.POST('/api/public/auth/signup', { body: { username, password } }).then(v => v.data),
    login: (username: string, password: string, otp?: string) =>
        client.POST('/api/public/auth/login', { body: { username, password, otp } }).then(v => v.data),

    send_recovery_key: (username: string) =>
        client.POST('/api/public/auth/send_recovery_key', { body: username }).then(v => v.data),
    reset_password: (key: string, password: string) =>
        client.POST('/api/public/auth/reset_password', { body: { key, password } }).then(v => v.data),


    change_password: (oldPassword: string, newPassword: string) =>
        client.POST('/api/user/auth/change_password', { body: { oldPassword, newPassword } }).then(v => v.data),

    logout: () => client.POST('/api/user/auth/logout').then(v => v.data),
    logoutAll: () => client.POST('/api/user/auth/logout_all').then(v => v.data),
    deleteAccount: () => client.POST('/api/user/auth/delete').then(v => v.data),
    sessionInfo: () => client.POST('/api/user/session').then(v => v.data),

    tfaSetupMail: () => client.POST('/api/user/tfa/setup_mail').then(v => v.data),
    tfaSetupTotp: () => client.POST('/api/user/tfa/setup_totp').then(v => v.data),
    tfaComplete: (code: string) => client.POST('/api/user/tfa/complete', { body: code }).then(v => v.data),
    tfaDisable: () => client.POST('/api/user/tfa/disable').then(v => v.data),


    getNode: (node: number) => client.POST('/api/user/fs/node', { body: node }).then(v => v.data),
    getPath: (node: number) => client.POST('/api/user/fs/path', { body: node }).then(v => v.data),
    getNodesSize: (nodes: number[]) => client.POST('/api/user/fs/size', { body: nodes }).then(v => v.data),
    getMime: (node: number) => client.POST('/api/user/fs/mime', { body: node }).then(v => v.data),
    downloadPreview: (node: number) => client.POST('/api/user/fs/preview', { body: node }).then(v => v.data),

    createNode: (name: string, parent: number, file: boolean) =>
        client.POST('/api/user/fs/create', { body: { name, parent, file } }).then(v => v.data),

    deleteNodes: (nodes: number[], cbk: (v: string|null) => void) => fetchEventSource('/api/user/fs/delete', {
        method: 'POST',
        body: JSON.stringify(nodes),
        headers: {
            'Authorization': 'Bearer ' + rpc.token,
            'Content-Type': 'application/json'
        },
        onmessage: v => cbk(v.data),
        onerror: e => { throw e; },
        onclose: () => cbk(null)
    }),

    admin: {
        listUsers: () => client.POST('/api/admin/users').then(v => v.data),
        setEnabled: (id: number, state: boolean) => client.POST('/api/admin/user/set_enabled', { body: { id, state } }).then(v => v.data),
        setAdmin: (id: number, state: boolean) => client.POST('/api/admin/user/set_admin', { body: { id, state } }).then(v => v.data),
        sudo: (id: number) => client.POST('/api/admin/user/sudo', { body: id }).then(v => v.data),
        logout: (id: number) => client.POST('/api/admin/user/logout', { body: id }).then(v => v.data),
        disableTfa: (id: number) => client.POST('/api/admin/user/disable_tfa', { body: id }).then(v => v.data),
        deleteUser: (id: number) => client.POST('/api/admin/user/delete', { body: id }).then(v => v.data),
        unSudo: () => client.POST('/api/admin/un_sudo').then(v => v.data),
        shutdown: () => client.POST('/api/admin/shutdown').then(v => v.data),
    }
};
