package de.mattv.fileserver.services;

import de.mattv.fileserver.Utils;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.util.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.time.Instant;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

@Slf4j
@Service
public class TFAMailer {
    private static final ConcurrentHashMap<String, Pair<Long, Instant>> MAIL_OTP = new ConcurrentHashMap<>();

    private static final Duration LIFETIME = Duration.ofMinutes(5);
    private static final int CODE_LENGTH = 10;

    private final Mailer mailer;

    public TFAMailer(@NonNull Mailer mailer) {
        this.mailer = mailer;
    }

    public void sendMail(@NonNull User user) {
        log.info("Sending tfa mail to: {}", user.name);
        String code = RandomStringUtils.random(CODE_LENGTH, false, true);
        MAIL_OTP.put(code, Pair.of(user.id, Instant.now().plus(LIFETIME)));
        mailer.send(user.name, "TFA Code", "Your code is: " + code + "\nIt is valid for " + LIFETIME.toMinutes() + " minutes.");
    }

    public boolean checkCode(long userId, @NonNull String code) {
        if (code.length() != CODE_LENGTH)
            return false;
        Pair<Long, Instant> entry = MAIL_OTP.remove(code);
        if (entry == null || Utils.instantExpired(entry.getRight()))
            return false;
        if (entry.getLeft() != userId)
            MAIL_OTP.put(code, entry);
        return entry.getLeft() == userId;
    }

    @Scheduled(fixedRate = 2, timeUnit = TimeUnit.HOURS)
    private static void cleanExpiredKeys() {
        MAIL_OTP.entrySet().removeIf(entry -> Utils.instantExpired(entry.getValue().getRight()));
    }
}
